Module plotting

Module plotting 

Functions

plot_values
Plot any slice of f64 values with automatic dynamic scaling
plot_values_with_range
Plot with custom Y-axis range (useful when you want to set your own bounds) Note: textplots doesn’t support custom Y ranges, so this clips the data to the range