ScanPropsBuilder

Struct ScanPropsBuilder 

pub struct ScanPropsBuilder {
    pub continuous_scan: Option<bool>,
    pub bouncy_scan: Option<bool>,
    pub autosave: Option<AutosaveMode>,
    pub series_name: Option<String>,
    pub comment: Option<String>,
    pub modules_names: Option<Vec<String>>,
    pub autopaste: Option<AutopasteMode>,
}
Builder for setting scan properties. Use None for fields that should not be changed.

Fields

continuous_scan: Option<bool>

Continuous scan: None = no change, Some(true) = On, Some(false) = Off

bouncy_scan: Option<bool>

Bouncy scan: None = no change, Some(true) = On, Some(false) = Off

autosave: Option<AutosaveMode>

Autosave mode: None = no change

series_name: Option<String>

Base name for saved images: None = no change

comment: Option<String>

Comment saved in file: None = no change

modules_names: Option<Vec<String>>

Module names whose parameters are saved in image header: None = no change

autopaste: Option<AutopasteMode>

Autopaste mode: None = no change

Implementations

Source

impl ScanPropsBuilder

pub fn new() -> ScanPropsBuilder

Create a new builder with all fields set to None (no changes)

pub fn continuous_scan(self, value: bool) -> ScanPropsBuilder

Set continuous scan mode

pub fn bouncy_scan(self, value: bool) -> ScanPropsBuilder

Set bouncy scan mode

pub fn autosave(self, mode: AutosaveMode) -> ScanPropsBuilder

Set autosave mode

pub fn series_name(self, name: impl Into<String>) -> ScanPropsBuilder

Set series name

pub fn comment(self, comment: impl Into<String>) -> ScanPropsBuilder

Set comment

pub fn modules_names(self, names: Vec<String>) -> ScanPropsBuilder

Set modules names

pub fn autopaste(self, mode: AutopasteMode) -> ScanPropsBuilder

Set autopaste mode