Functions
- plot_
values - Plot any slice of f64 values with automatic dynamic scaling
- plot_
values_ with_ range - Plot with custom Y-axis range (useful when you want to set your own bounds) Note: textplots doesn’t support custom Y ranges, so this clips the data to the range